# Copyright 2014 Baidu, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License") you may not use this file
# except in compliance with the License. You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software distributed under the
# License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
# either express or implied. See the License for the specific language governing permissions
# and limitations under the License.

"""
The type of KeySpec
"""

BAIDU_AES_256 = 'BAIDU_AES_256'
AES_128 = 'AES_128'
AES_256 = 'AES_256'
SM1_128 = 'SM1_128'
SM2_256 = 'SM2_256'
SM4_128 = 'SM4_128'
RSA_1024 = 'RSA_1024'
RSA_2048 = 'RSA_2048'
RSA_4096 = 'RSA_4096'
